
from zope.interface import implements
from zope.component import getMultiAdapter

from interfaces import IRuleOverwrite 

def beforeTraverseHandler(context, event):
    """ This handler is the main working horse and checks if
        public skin should be enabled. That means marking the
        request with IPublicSkinLayer. """

    skinner = getMultiAdapter( (context, event.request), name=u'skinner')
    if skinner.mustDisplayPublicView() is True:
        skinner.activatePublicSkin()



from zope import interface
from zope.publisher.interfaces.browser import IDefaultBrowserLayer

from browser.interfaces import *

class IPublicSkinLayer(IDefaultBrowserLayer):
    """ Layer for the public view """

class IRuleOverwrite(interface.Interface):
    """ Interface to make rule overwrite possible """

    def mustDisplayPublicView(context, request):
        pass




from Acquisition import aq_inner, aq_parent, aq_base

from zope.component import ComponentLookupError
from zope.interface import implements, directlyProvides

from Products.Five import BrowserView
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.interfaces import IFolderish

from anthill.skinner.interfaces import IPublicSkinLayer, IRuleOverwrite

from anthill.skinner.config import VIEW_CMS_PERMISSION, \
                                        PREVIEW_MODE, \
                                            RQ_PREVIEW_VAR

from interfaces import ISkinHandler

class SkinHandling(BrowserView):
    """ @see ISkinHandler """

    implements(ISkinHandler)

    def __init__(self, context, request):
        BrowserView.__init__(self, context, request)
        self.pm = getToolByName(self.context, 'portal_membership')

        # provide folderish context
        self.folder = aq_inner(self.context)
        if not IFolderish.providedBy(self.folder):
            self.folder = aq_parent(self.folder)

    def _provideAuth(self):
        # before traversing zope hasn't yet authenticated the user
        # so we need to authenticate user here - this relies
        # on code found in BaseRequest.py(424ff) and can break in future
        # releases (but is unlikely) of Plone or Zope.
        # This code performs more or less the same steps as Zope does
        # to authenticate user so blame others for this :-)

        # first we need to find all acl_users folders
        acls = []
        for ob in self.request['PARENTS']:

            # aq-aware search
            if hasattr(ob, 'acl_users'):
                acls.append(getattr(ob, 'acl_users'))

            # also support having acl_users folder in chain
            if hasattr(aq_base(ob), 'id') and ob.id == 'acl_users':
                acls.append(ob)

        # we absolutely need at least two acl_users folder
        # one for the plone site and one for the app root
        if len(acls) >= 2:

            # hack the request to hide the fact that we don't
            # have anything published yet
            self.request['PUBLISHED'] = self.folder

            for acl in acls:
                user = acl.validate(self.request, self.request._auth)
                if user:
                    self.request['AUTHENTICATED_USER'] = user
                    return user

            return None

        else:
            raise Exception, 'Error looking up auth validation provider!'

    def _removeAuth(self):
        # removes authentication if performed by _provideAuth
        if self.request.has_key('PUBLISHED'):
            del self.request.__dict__['other']['PUBLISHED']
            del self.request.__dict__['other']['AUTHENTICATED_USER']

    def _isAnonymous(self):
        isanon = self.pm.isAnonymousUser()

        # there can be cases where user seems unauthed
        # http://mail.zope.org/pipermail/zope/2001-May/090253.html
        if isanon and \
                (self.request.cookies.has_key('__ac') or \
                 hasattr(self.request, '_auth')):
                
                # try to auth user so that we can check permissions
                if self._provideAuth() != None:
                    return False

        return isanon and True or False

    def _hasViewCMSPermission(self): 
        if not self._isAnonymous():
            checkPermission = getToolByName(self.context,
                                'portal_membership').checkPermission
            ok = checkPermission(VIEW_CMS_PERMISSION, self.context)

            # allow fallback to container
            if not ok and self.folder != self.context:
                ok = checkPermission(VIEW_CMS_PERMISSION, self.folder)
            return ok and True or False

        else: 
            return False

    def _redirectToCaller(self):
        return self.request.RESPONSE.redirect(self.folder.absolute_url())

    def mustDisplayPublicView(self):
        """ @see ISkinHandler """

        # check adapter first if available
        try:
            adapted = IRuleOverwrite(self)
            return adapted.mustDisplayPublicView(self.context, self.request)
        except (ComponentLookupError, TypeError):
            pass

        doit = self._isAnonymous() or \
                not self._hasViewCMSPermission() or \
                 self.inPreview()

        self._removeAuth()
        return doit

    def activatePreview(self):
        """ @see ISkinHandler """

        if not self.inPreview() and not self._isAnonymous():
            self.request.SESSION.set(PREVIEW_MODE, True)
            self._removeAuth()
            self._redirectToCaller()

    def deactivatePreview(self):
        """ @see ISkinHandler """

        if self.inPreview():
            self.request.SESSION.set(PREVIEW_MODE, False)
            self._removeAuth()
            self._redirectToCaller()

    def inPreview(self):
        """ @see ISkinHandler """

        if not self._isAnonymous():
            self._removeAuth()
            return self.request.SESSION.get(PREVIEW_MODE, False) or \
                    self.request.form.has_key(RQ_PREVIEW_VAR)

    def inPublicView(self):
        """ @see ISkinHandler """

        return self.mustDisplayPublicView()

    def activatePublicSkin(self):
        self.context.changeSkin('publicview', self.request)
        directlyProvides(self.request, IPublicSkinLayer)





import zope.schema
from zope.interface import Interface, directlyProvides
from zope.contentprovider.interfaces import ITALNamespaceData

class ISkinHandler(Interface):
    """ Handler that provides functionality to switch between
        public view and plone default skin. """

    def activatePreview():
        """ Switches skin to public mode """

    def deactivatePreview():
        """ Switches skin back to default """

    def inPreview():
        """ Returns True if currently in preview mode """

    def inPublicView():
        """ Returns True if system is in public view. Will also return
            True if only in preview mode. """

    def activatePublicSkin():
        """ Activates public skin by setting appropriate layers """

    def mustDisplayPublicView():
        """ Returns True if public view has to be displayed """

class INavigationBase(Interface):
    """ Simple view providing methods to use path bar and section top
        viewlets (from plone). If you want completely customized navigation then
        use INavigationExtended implementation. """

    def renderNavigation():
        pass

    def renderPathBar():
        pass

    def renderBackToCMSLink():
        pass

    def menuIsActive(topLevel, bottomLevel):
        pass

class ICustomMenuParameters(Interface):
    """ Holds available parameters for the custom menu content provider """

    enclosing_tag = zope.schema.Text(title=u'The enclosing tag for menu generation',
                                     default=u'ul')

    topLevel = zope.schema.Int(title=u'The level of the items to start at - starts at 1',
                                default=1)

    bottomLevel = zope.schema.Int(title=u'The level of the items to stop at',
                                    default=65536)

    includeTop = zope.schema.Bool(title=u'Should we include top level folder',
                                     default=True)

# we need to mark this interface to make the implementing class accept params
directlyProvides(ICustomMenuParameters,
                 ITALNamespaceData)



from setuptools import setup, find_packages
import os

version = '0.1'

setup(name='anthill.skinner',
      version=version,
      description="Skinning for plone made easy",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='skinning theming plone zope',
      author='Simon Pamies',
      author_email='s.pamies@banality.de',
      url='http://www.banality.de',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['anthill'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'anthill.tal.macrorenderer',
          'collective.autopermission',
      ],
      entry_points="""
      # -*- Entry points: -*-

      [distutils.setup_keywords]
      paster_plugins = setuptools.dist:assert_string_list

      [egg_info.writers]
      paster_plugins.txt = setuptools.command.egg_info:write_arg
      """,
      paster_plugins = ["ZopeSkel"],
      )




from zope.interface import implements
from Acquisition import aq_inner, aq_parent
from zope.component import queryMultiAdapter

from zope.contentprovider.interfaces import IContentProvider

from Products.Five.browser import BrowserView
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.interfaces import IFolderish

import plone.app.layout.viewlets.common
import plone.app.portlets.portlets.navigation

from anthill.tal.macrorenderer import MacroRenderer

from interfaces import INavigationBase, ICustomMenuParameters

class NavigationBase(BrowserView):
    """ @see: INavigationBase """

    implements(INavigationBase)

    def __init__(self, context, request):
        self.context = aq_inner(context)
        self.request = request

    def _render_viewlet(self, factory):
        # shamelessly copied from collective.skinny
        viewlet = factory(self.context, self.request, None, None).__of__(self.context)
        viewlet.update()
        return viewlet.render()

    def renderPathBar(self):
        return self._render_viewlet(plone.app.layout.viewlets.common.PathBarViewlet)

    def renderNavigation(self):
        return self._render_viewlet(plone.app.layout.viewlets.common.GlobalSectionsViewlet)

    def renderBackToCMSLink(self):
        return '<a href="%s/@@skinner/deactivatePreview" i18n:translate="">Back to Plone</a>' % self.context.absolute_url()

    def _folderishContextOf(self, ct):
        folder = aq_inner(ct)
        if not IFolderish.providedBy(folder):
            folder = aq_parent(folder)
        return folder

    def menuIsActive(self, topLevel, bottomLevel):
        # algorithm is as follows:
        # - get the plone root path
        # - get the context path (folderish)
        # - check if current level of path == topLevel - 1 and < bottomLevel 
        # - check if there are any items displayable

        purl = getToolByName(self.context, 'portal_url')
        portal = aq_inner(purl.getPortalObject())

        # shortcuts
        if topLevel <= 0:
            return True

        # submenu of portal should always be active
        _fd = self._folderishContextOf(self.context)
        if _fd is portal:
            return False

        portalpath = aq_inner(portal).getPhysicalPath()
        currentpath = _fd.getPhysicalPath()
        level = len(currentpath[len(portalpath):])

        levelok = (level >= (topLevel) and level <= bottomLevel)

        # it is a bit tricky to determine if there are items to be displayed
        # because we can't rely on a simple getFolderContents or such. Instead
        # we need to load the navtree engine and then check if there are any items
        # provided. This is a bit slow but there is no other way. We use the data
        # provided by PublicMenu here.
        cnav = queryMultiAdapter( (self.context, self.request, self), 
                                    name = u'anthill.skinner.PublicMenu')
        cnav.update()
        navok = len(cnav.data['children']) > 0

        return levelok and navok

class CustomNavigation(object):
    """ Custom navigation that relies on standard plone engine but simplifies life a bit
        by using templates diectly accessible by the user. """

    implements(ICustomMenuParameters, IContentProvider)

    enclosing_tag = None
    includeTop = True
    topLevel = 1
    bottomLevel = 65536

    def __init__(self, context, request, view):
        self.context = aq_inner(context)
        self.request = request
        self.__parent__ = view

    def update(self):
        # we use Renderer directly here (querying multi adapter would be too
        # complicated for a portlet) in order to mimic the navtree portlet
        # as much as possible

        data = plone.app.portlets.portlets.navigation.Assignment('publicmenu',
                    currentFolderOnly=False, includeTop=self.includeTop,
                    topLevel=self.topLevel, bottomLevel=self.bottomLevel)

        self.navrenderer = plone.app.portlets.portlets.navigation.Renderer(
                                            self.context, self.request,
                                            view=None, manager=None, 
                                            data=data)

        self.data = self.navrenderer.getNavTree()
        self.navroot = self.navrenderer.getNavRoot()

    def _computeMacroName(self, level, template):
        # we compute the macro name to be used. This is needed
        # because macro levels can be 'acquired'. If there is for
        # example no macro for level n we search for a def for n-1

        levels = range(level); levels.reverse();
        for n in levels:
            try:
                name = 'level_%s' % int(n)
                template.macros[name]
                return name
            except KeyError:
                continue

        raise Exception, 'You seem to have forgotten to define macro levels!'

    def _recurse(self, template, level, node):
        output = u''
        if level <= 0 or level <= self.bottomLevel:
            children = node.get('children', [])
            for child in children:
                data = {'item' : child,
                        'isGlobalCurrent' : child['currentItem']}

                renderer = MacroRenderer(template, self._computeMacroName(level, template))
                output += renderer(data=data)

                if len(child['children']) > 0 and child['show_children']:
                    output += self._recurse(template, level+1, child)

        return output

    def render(self):

        # load template (residing in portal_skins to ease customization)
        navtemplate = self.context.unrestrictedTraverse('publicmenu_levels')

        struct = {}
        for name in ['include_top', 'root_item_class', 'root_is_portal', 'navigation_root']:
            struct[name] = getattr(self.navrenderer, name)
       
        defout = u'%s'; output = u''
        if self.enclosing_tag != None:
            defout = '<%(tag)s>%%s<%(tag)s>' % {'tag' : self.enclosing_tag} 

        # render root item first and then children
        output += MacroRenderer(navtemplate, 'rootitem')(data=struct)
        output += self._recurse(navtemplate, level=1, node=self.data)

        return defout % output

